from django.urls import path
from filehub.views import *

app_name = 'filehub'
urlpatterns = [
    path('filehub/', browser_view, name='browser'),
    path('filehub/select/', browser_select, name='browser_select'),
    path('filehub/ajax/browse/', browser_ajax, name='browser_ajax'),
    path('filehub/ajax/folder/', NewFolderView.as_view(), name='new_folder'),
    path('filehub/ajax/delete/', delete_folder, name='delete_folder'),
    path('filehub/ajax/upload/', upload_file, name='upload')
]



import os
from django.conf import settings

FILEMANAGER_VERSION = "3.2.0"

FILEHUB_LOGIN_URL = getattr(settings, 'FILEHUB_LOGIN_URL', '/admin/')

DIRECTORY = getattr(settings, 'FILEMANAGER_DIRECTORY', 'uploads')
THUMB_DIRECTORY = getattr(settings, 'THUMB_DIRECTORY', 'thumbs')

MEDIA_ROOT = getattr(settings, "FILEMANAGER_MEDIA_ROOT", DIRECTORY + "/")
MEDIA_URL = getattr(settings, "FILEMANAGER_MEDIA_URL", os.path.join(settings.MEDIA_URL, DIRECTORY))
UPLOAD_DIRECTORY_URL = MEDIA_URL + "/"

FILE_TYPE_CATEGORIES = getattr(settings, "FILE_TYPE_CATEGORIES", {
    'images': ['jpg', 'jpeg', 'png', 'gif', 'bmp', 'tiff', 'webp', 'svg', 'ico'],
    'videos': ['mp4', 'webm', 'ogg', 'avi', 'mkv', 'mov', 'wmv', '3gp', 'mpeg', 'mpg4'],
    'musics': ['mp3', 'wav', 'flac', 'aac', 'wma', 'm4a', 'wma'],
    'archives': ['zip', 'rar', 'tar', 'gz'],
    'documents': ['pdf', 'doc', 'docx', 'xls', 'xlsx', 'ppt', 'pptx', 'txt', 'csv', 'odt', 'ods'],
})

FILES_SORTING = getattr(settings, "FILES_SORTING", {
    "name": "Name",
    "size": "Size",
    "date": "Modified"
})

FILES_SORTING_ORDER = getattr(settings, "FILES_SORTING_ORDER", {
    "asc": "Ascending",
    "desc": "Descending"
})

EMPTY_FOLDER_SIZE = getattr(settings, "EMPTY_FOLDER_SIZE", 1024)
STATIC_PATH = getattr(settings, "STATIC_PATH", os.path.join(settings.BASE_DIR, "static"))
STATIC_URL = getattr(settings, "STATIC_URL", "static/")
FILEHUB_THEME_COLOR = getattr(settings, "FILEHUB_THEME_COLOR", "#009688")

FILEHUB_ACCESS_MODE = getattr(settings, "FILEHUB_ACCESS_MODE", 'all')
FILEHUB_ROLE_FIELD = getattr(settings, "FILEHUB_ROLE_FIELD", 'role')
FILEHUB_AUTO_CLOSE_UPLOAD_MODAL = getattr(settings, "FILEHUB_AUTO_CLOSE_UPLOAD_MODAL", False)




from django import template

register = template.Library()


@register.filter
def hex_to_rgb(value):
    """Convert a hex color code to RGB"""
    if value.startswith('#'):
        value = value[1:]

    if len(value) == 6:
        r = int(value[0:2], 16)
        g = int(value[2:4], 16)
        b = int(value[4:6], 16)
        return f'{r}, {g}, {b}'
    return value



import warnings

from django.contrib.admin import widgets as admin_widgets
from django.db import models
from filehub import widgets as filehub_widgets
from filehub.forms import GalleryFormField, FilePickerFormField
from filehub.settings import FILE_TYPE_CATEGORIES


class ImagePickerField(models.TextField):
    """
    A string field for storing image URLs. This field uses the `ImagePickerWidget`
    in forms to allow users to select images through a file manager interface.

    The widget allows users to pick an image from the file manager, which will
    then insert the image URL into the field. In the Django admin, it uses
    `AdminImagePickerWidget` for a more integrated interface.
    """

    def __init__(self, *args, file_type=None, file_ext=None, **kwargs):
        self.file_type = file_type
        self.file_ext = file_ext
        super().__init__(*args, **kwargs)

    def deconstruct(self):
        name, path, args, kwargs = super().deconstruct()
        kwargs['file_type'] = self.file_type
        kwargs['file_ext'] = self.file_ext
        return name, path, args, kwargs

    def formfield(self, **kwargs):
        defaults = {"widget": filehub_widgets.ImagePickerWidget}
        defaults.update(kwargs)

        # As an ugly hack, we override the admin widget
        if defaults["widget"] == admin_widgets.AdminTextareaWidget:
            defaults["widget"] = filehub_widgets.AdminImagePickerWidget

        if self.file_type:
            if not isinstance(self.file_type, list):
                raise ValueError("file_type must be a list of allowed file types")

        if self.file_ext:
            if not isinstance(self.file_ext, list):
                raise ValueError("file_ext must be a list of allowed file extensions")

        field = super().formfield(**defaults)
        field.widget.attrs.update({
            "data-file-type": ",".join(self.file_type) if self.file_type else "",
            "data-file-ext": ",".join(self.file_ext) if self.file_ext else "",
        })
        return field


class FilePickerField(models.JSONField):
    description = "Custom JSON field for picking a single file"

    def __init__(self, *args, **kwargs):
        self.max_length = kwargs.pop('max_length', None)
        if self.max_length is not None:
            warnings.warn(
                "`max_length` has no effect on FilePickerField.",
                stacklevel=2
            )

        file_type = kwargs.pop("file_type", [])
        file_ext = kwargs.pop("file_ext", [])

        if isinstance(file_type, str):
            file_type = [file_type]
        if isinstance(file_ext, str):
            file_ext = [file_ext]

        for t in file_type:
            if t not in FILE_TYPE_CATEGORIES:
                raise ValueError(f"Invalid file_type '{t}'. Allowed keys: {list(FILE_TYPE_CATEGORIES.keys())}")

        all_allowed_exts = set(
            ext.lower() for exts in FILE_TYPE_CATEGORIES.values() for ext in exts
        )

        for ext in file_ext:
            ext_clean = ext.lower().strip()
            if ext_clean not in all_allowed_exts:
                raise ValueError(
                    f"Invalid file extension '{ext}'. Must be one of the allowed extensions in FILE_TYPE_CATEGORIES.")

        extensions_from_types = []
        for t in file_type:
            extensions_from_types.extend(FILE_TYPE_CATEGORIES[t])

        merged_exts = set(ext.lower().strip() for ext in extensions_from_types + file_ext)
        self.file_ext = sorted(merged_exts)
        super().__init__(*args, **kwargs)

    def formfield(self, **kwargs):
        defaults = {
            'form_class': FilePickerFormField,
            'file_ext': self.file_ext,
        }
        defaults.update(kwargs)
        return super().formfield(**defaults)


class GalleryPickerField(models.JSONField):
    description = "Custom JSON field for picking an image gallery (list of images)"

    def __init__(self, *args, **kwargs):
        kwargs.setdefault('default', list)
        kwargs.setdefault('blank', True)
        self.min_items = kwargs.pop("min_items", None)
        self.min_items = kwargs.pop("min_items", None)
        self.max_items = kwargs.pop("max_items", None)
        self.sortable = kwargs.pop("sortable", False)

        self.max_length = kwargs.pop('max_length', None)
        if self.max_length is not None:
            warnings.warn(
                "`max_length` has no effect on GalleryPickerField. Use `max_items` instead.",
                stacklevel=2
            )

        super().__init__(*args, **kwargs)

    def formfield(self, **kwargs):
        defaults = {
            'form_class': GalleryFormField,
            'min_items': self.min_items,
            'max_items': self.max_items,
            'sortable': self.sortable,
        }
        defaults.update(kwargs)
        return super().formfield(**defaults)





import os
from django.http import JsonResponse
from django.shortcuts import render
from django.core.paginator import Paginator, PageNotAnInteger, EmptyPage
from django.utils.decorators import method_decorator
from django.views import View
from django.views.decorators.csrf import csrf_exempt
from django.contrib.auth.decorators import login_required
from django.db.models import Q

from filehub.models import MediaFolder, MediaFile
from filehub.core import FolderManager
from filehub.settings import (
    MEDIA_URL, FILES_SORTING, FILES_SORTING_ORDER, FILE_TYPE_CATEGORIES,
    FILEHUB_LOGIN_URL, FILEHUB_THEME_COLOR, FILEMANAGER_VERSION,
    FILEHUB_AUTO_CLOSE_UPLOAD_MODAL
)
from filehub.utils import file_response_format, folder_response_format, sanitize_allowed_extensions


def _context_base(select_files=True, select_multiple=True, file_picker=False, file_exts=None, cleaner_mode=False):
    """Return the common template context for all FileHub views."""
    return {
        "media_url": MEDIA_URL,
        "sorting": {"order": FILES_SORTING_ORDER, "sort": FILES_SORTING},
        "allowed_exts": FILE_TYPE_CATEGORIES,
        "select_files": str(select_files).lower(),
        "select_multiple": str(select_multiple).lower(),
        "file_picker": file_picker,
        "theme_color": FILEHUB_THEME_COLOR,
        "version": FILEMANAGER_VERSION,
        "file_exts": file_exts or [],
        "cleaner_mode": cleaner_mode,
        "auto_close_upload_modal": "true" if FILEHUB_AUTO_CLOSE_UPLOAD_MODAL else "false",
    }


def get_folder_hierarchy(parent_folder=None):
    """Recursively build the folder hierarchy starting from the given parent."""
    folders = MediaFolder.objects.filter(parent=parent_folder).order_by("folder_name")
    return [
        {
            "id": f.id,
            "name": f.folder_name,
            "path": f.get_relative_path(),
            "uri": f.get_relative_path(),
            "children": get_folder_hierarchy(f),
        }
        for f in folders
    ]


def get_folders(parent_folder=None, sortby="name", sortorder="asc", search=None, request=None):
    """Return a list of folders, sorted and optionally filtered by search query."""
    sort_prefix = "" if sortorder == "asc" else "-"
    if request and getattr(request.user, "is_authenticated", False):
        folders = MediaFolder.get_accessible_folders(request.user, parent_folder)
    else:
        folders = MediaFolder.objects.filter(parent=parent_folder)

    if search:
        folders = folders.filter(Q(folder_name__icontains=search))

    if sortby == "name":
        folders = folders.order_by(f"{sort_prefix}folder_name")
    elif sortby == "date":
        folders = folders.order_by(f"{sort_prefix}modify_date")

    result = [folder_response_format(f, request) for f in folders]
    if sortby == "size":
        result = sorted(result, key=lambda x: x["size"], reverse=(sortorder == "desc"))
    return result


def get_files(page=1, folder=None, sortby="name", sortorder="asc", search=None, filter_by=None, request=None):
    """Return a paginated and formatted list of files for a given folder."""
    sort_prefix = "" if sortorder == "asc" else "-"
    sort_map = {"name": "file_name", "date": "modify_date", "size": "file_size"}
    sort_field = sort_map.get(sortby, "file_name")

    if request and getattr(request.user, "is_authenticated", False):
        all_files = MediaFile.get_accessible_files(request.user, folder)
    else:
        all_files = MediaFile.objects.filter(folder=folder)

    if filter_by:
        all_files = all_files.filter(file_type=filter_by)
    if search:
        all_files = all_files.filter(Q(file_name__icontains=search))

    paginator = Paginator(all_files.order_by(f"{sort_prefix}{sort_field}"), 49)

    try:
        current_page = paginator.page(page)
    except (PageNotAnInteger, EmptyPage):
        current_page = paginator.page(1)

    return {
        "hasMore": current_page.has_next(),
        "data": [file_response_format(f, request) for f in current_page.object_list if f],
    }


@login_required(login_url=FILEHUB_LOGIN_URL)
def browser_view(request):
    """Render the main FileHub browser view."""
    return render(request, "filehub/filehub_list.html", context=_context_base())


@login_required(login_url=FILEHUB_LOGIN_URL)
def browser_select(request):
    """Render the FileHub view for file selection (picker) mode."""
    file_exts = sanitize_allowed_extensions(request.GET.getlist("file_ext", []))
    multiple = request.GET.get("multiple", "false").lower() == "true"
    return render(
        request,
        "filehub/filehub_list.html",
        context=_context_base(file_picker=True, select_multiple=multiple, file_exts=file_exts),
    )


@login_required(login_url=FILEHUB_LOGIN_URL)
def file_cleaner_view(request):
    """Render the file cleaner view for removing unused or orphaned files."""
    return render(request, "filehub/file_cleaner.html", context=_context_base(cleaner_mode=True))


@login_required(login_url=FILEHUB_LOGIN_URL)
@csrf_exempt
def clear_selected_files(request):
    """Delete selected files and folders via POST request."""
    if request.method != "POST":
        return JsonResponse({"success": False, "message": "Invalid request method"}, status=400)

    file_ids = request.POST.getlist("file_ids[]", [])
    folder_ids = request.POST.getlist("folder_ids[]", [])
    if not file_ids and not folder_ids:
        return JsonResponse({"success": False, "message": "No files or folders selected"}, status=400)

    cleared_files = MediaFile.objects.filter(id__in=file_ids).count()
    MediaFile.objects.filter(id__in=file_ids).delete()

    cleared_folders = MediaFolder.objects.filter(id__in=folder_ids).count()
    MediaFolder.objects.filter(id__in=folder_ids).delete()

    return JsonResponse({
        "success": True,
        "message": f"Cleared {cleared_files} files and {cleared_folders} folders",
        "cleared_files": cleared_files,
        "cleared_folders": cleared_folders,
    })


@login_required(login_url=FILEHUB_LOGIN_URL)
@csrf_exempt
def browser_ajax(request):
    """Handle AJAX requests for listing folders and files with pagination and sorting."""
    if request.method != "POST":
        return JsonResponse({"success": False, "message": "Invalid request method"}, status=400)

    try:
        folder_id = request.POST.get("folder")
        folder = MediaFolder.objects.filter(id=folder_id).first() if folder_id else None
        breadcrumb = folder.get_breadcrumb() if folder else {}

        folders = get_folders(folder, request.POST.get("sortby", "name"), request.POST.get("sortorder", "asc"),
                              request.POST.get("search"), request)
        files = get_files(
            page=request.POST.get("page", 1),
            folder=folder,
            sortby=request.POST.get("sortby", "name"),
            sortorder=request.POST.get("sortorder", "asc"),
            search=request.POST.get("search"),
            filter_by=request.POST.get("filter"),
            request=request,
        )

        return JsonResponse({
            "success": True,
            "message": "Data fetched successfully",
            "hiearchy": get_folder_hierarchy(),
            "folders": folders,
            "files": files,
            "folder": folder_id,
            "page": request.POST.get("page", 1),
            "breadcrumb": breadcrumb,
        })

    except (ValueError, TypeError) as e:
        return JsonResponse({"success": False, "message": str(e)}, status=400)
    except Exception as e:
        return JsonResponse({"success": False, "message": "Internal server error", "error": str(e)}, status=500)


@method_decorator([csrf_exempt, login_required(login_url=FILEHUB_LOGIN_URL)], name="dispatch")
class NewFolderView(View):
    """Create or rename folders and files."""

    def post(self, request, *args, **kwargs):
        """Handle POST request for creating or renaming a folder or file."""
        file_type = request.POST.get("file_type", "folder")
        try:
            return (
                self._handle_folder_request(request)
                if file_type == "folder"
                else self._handle_file_request(request)
            )
        except MediaFolder.DoesNotExist:
            return JsonResponse({"success": False, "message": "Folder does not exist"}, status=404)
        except MediaFile.DoesNotExist:
            return JsonResponse({"success": False, "message": "File does not exist"}, status=404)
        except Exception as e:
            return JsonResponse({"success": False, "message": str(e)}, status=400)

    def _handle_folder_request(self, request):
        """Handle folder create or rename request."""
        update_id = request.POST.get("update_id")
        name = request.POST.get("name")
        if not name:
            return JsonResponse({"success": False, "message": "Folder name is required"}, status=400)

        if update_id:
            folder = MediaFolder.objects.get(id=update_id)
            folder.folder_name = name
            folder.save()
            return JsonResponse({
                "success": True,
                "message": "Folder renamed successfully",
                "data": folder_response_format(folder, request),
            })

        parent_id = request.POST.get("current_dir")
        parent = MediaFolder.objects.filter(id=parent_id).first() if parent_id else None
        if MediaFolder.objects.filter(folder_name=name, parent=parent).exists():
            return JsonResponse({"success": False, "message": "Folder already exists"}, status=400)

        new_folder = MediaFolder(folder_name=name, parent=parent, upload_by=request.user)
        new_folder.save()
        FolderManager.create_folder(new_folder)
        return JsonResponse({
            "success": True,
            "message": "Folder created successfully",
            "data": folder_response_format(new_folder, request),
        })

    def _handle_file_request(self, request):
        """Handle file rename request."""
        update_id = request.POST.get("update_id")
        name = request.POST.get("name")

        if not update_id or not name:
            return JsonResponse({"success": False, "message": "Missing required fields"}, status=400)

        file_obj = MediaFile.objects.get(id=update_id)
        base, ext = os.path.splitext(file_obj.file_name)
        file_obj.file_name = f"{name}{ext}"
        FolderManager.rename_file(file_obj, file_obj)
        file_obj.save()
        return JsonResponse({
            "success": True,
            "message": "File renamed successfully",
            "data": file_response_format(file_obj, request),
        })

    def get(self, request, *args, **kwargs):
        """Reject GET requests for this view."""
        return JsonResponse({"success": False, "message": "GET not allowed"}, status=400)


@login_required(login_url=FILEHUB_LOGIN_URL)
@csrf_exempt
def delete_folder(request):
    """Delete folders or files by ID list."""
    if request.method != "POST":
        return JsonResponse({"success": False, "message": "Invalid request method"}, status=400)

    delete_ids = request.POST.getlist("delete_id[]", [])
    if not delete_ids:
        return JsonResponse({"success": False, "message": "No IDs provided"}, status=400)

    file_type = request.POST.get("type", "folder")
    model = MediaFolder if file_type == "folder" else MediaFile
    qs = model.objects.filter(id__in=delete_ids)

    if not qs.exists():
        return JsonResponse({"success": False, "message": f"{file_type.title()} does not exist"}, status=404)

    count = qs.count()
    qs.delete()
    return JsonResponse({"success": True, "message": f"Deleted {count} {file_type}(s) successfully"})


@login_required(login_url=FILEHUB_LOGIN_URL)
@csrf_exempt
def upload_file(request):
    """Upload a new file into the specified folder."""
    if request.method != "POST":
        return JsonResponse({"success": False, "message": "Invalid request method"}, status=400)

    folder_id = request.POST.get("folder_id")
    file = request.FILES.get("file") or request.POST.get("url")

    if not file:
        return JsonResponse({"success": False, "message": "No file uploaded"}, status=400)

    try:
        folder = MediaFolder.objects.filter(id=folder_id).first() if folder_id else None
        media_file = FolderManager.upload_to_filemanager(file, folder, request.user)
        return JsonResponse({
            "success": True,
            "message": "File uploaded successfully",
            "data": file_response_format(media_file, request),
        })
    except Exception as e:
        return JsonResponse({"success": False, "message": str(e)}, status=400)



import os
from io import BytesIO

from PIL import Image
from django.core.files.storage import default_storage
from django.db import models

import filehub.settings
from filehub.settings import EMPTY_FOLDER_SIZE
from django.db.models import Sum, Value, IntegerField
from django.db.models.functions import Coalesce
from filehub.core import FolderManager
from django.core.exceptions import ValidationError
from django.conf import settings

from django.contrib.auth import get_user_model
User = get_user_model()


# Create your models here.
class MediaFolder(models.Model):
    folder_name = models.CharField(max_length=255)
    parent = models.ForeignKey("self", null=True, blank=True, on_delete=models.CASCADE)
    upload_by = models.ForeignKey(User, on_delete=models.SET_NULL, null=True, blank=True)
    upload_date = models.DateTimeField(auto_now_add=True, null=True)
    modify_date = models.DateTimeField(auto_now=True, null=True)

    def __str__(self):
        return self.folder_name

    def get_relative_path(self, new_folder_name=None):
        """
        Generate the relative file path for the MediaFolder instance based on its id.
        """
        folder_name = new_folder_name if new_folder_name is not None else ""
        relative_path = f"{self.id}/{folder_name}"
        return FolderManager.get_root_directory() + relative_path

    def get_full_path(self, folder_name=None):
        """
        Generate the full path for the MediaFolder instance based on its id.
        """
        relative_path = self.get_relative_path(folder_name)
        return os.path.join(settings.MEDIA_URL, relative_path)

    def get_breadcrumb(self):
        """
        Generate the breadcrumb path
        """
        output = []

        # Traverse the parent hierarchy to build the relative path
        current_folder = self
        while current_folder:
            output.append({
                "id": current_folder.id,
                "name": current_folder.folder_name
            })
            current_folder = current_folder.parent

        return output[::-1]

    def get_size(self) -> int:
        total_size = MediaFolder.objects.filter(parent=self).aggregate(
            size=Coalesce(
                Sum('mediafile__file_size', default=0),
                0, output_field=IntegerField()
            ) + Value(EMPTY_FOLDER_SIZE, output_field=IntegerField())
        ).get('size', 0)

        for child_folder in MediaFolder.objects.filter(parent=self):
            total_size += child_folder.get_size()

        return int(total_size)

    def delete(self, *args, **kwargs):
        try:
            FolderManager.delete_folder(self)
            super().delete(*args, **kwargs)
        except ValidationError as e:
            print(f"Error deleting MediaFolder instance: {str(e)}")

    @classmethod
    def get_accessible_folders(cls, user, parent_folder=None):
        """Get folders accessible to the user based on settings"""
        access_mode = filehub.settings.FILEHUB_ACCESS_MODE

        folders = cls.objects.filter(parent=parent_folder)

        if access_mode == 'own':
            folders = folders.filter(upload_by=user)

        elif access_mode == 'role_based':
            role_field = filehub.settings.FILEHUB_ROLE_FIELD
            if hasattr(user, role_field):
                user_role = getattr(user, role_field)
                folders = folders.filter(**{f'upload_by__{role_field}': user_role})
            else:
                folders = folders.filter(upload_by=user)
        elif access_mode == 'all':
            pass

        return folders



class MediaFile(models.Model):
    file_name = models.CharField(max_length=255)
    folder = models.ForeignKey(MediaFolder, on_delete=models.CASCADE, null=True, blank=True)
    file_type = models.CharField(max_length=50, blank=True, null=True)
    file_size = models.PositiveIntegerField(default=0, blank=True)
    width = models.PositiveIntegerField(default=0, blank=True)
    height = models.PositiveIntegerField(default=0, blank=True)
    upload_by = models.ForeignKey(User, on_delete=models.SET_NULL, null=True, blank=True)
    upload_date = models.DateTimeField(auto_now_add=True, null=True)
    modify_date = models.DateTimeField(auto_now=True, null=True)

    def __str__(self) -> str:
        return self.file_name

    @classmethod
    def get_accessible_files(cls, user, folder=None):
        access_mode = filehub.settings.FILEHUB_ACCESS_MODE

        files = cls.objects.filter(folder=folder)

        if access_mode == 'own':
            files = files.filter(upload_by=user)

        elif access_mode == 'role_based':
            role_field = filehub.settings.FILEHUB_ROLE_FIELD
            if hasattr(user, role_field):
                user_role = getattr(user, role_field)
                files = files.filter(**{f'upload_by__{role_field}': user_role})
            else:
                files = files.filter(upload_by=user)

        elif access_mode == 'all':
            pass

        return files

    def get_relative_path(self) -> str:
        if self.folder is None:
            return FolderManager.get_root_directory() + self.file_name
        return self.folder.get_relative_path(self.file_name)

    def get_full_path(self) -> str | bytes:
        if self.folder is None:
            return os.path.join(settings.MEDIA_URL, self.get_relative_path())
        return self.folder.get_full_path(self.file_name)

    def get_thumbnail_name(self) -> str:
        return f"{self.id}.jpg"

    def update_image_attributes(self, file=None):
        try:
            if self.file_type == 'image' or self.file_type == 'images':
                file_path = self.get_relative_path()

                thumb_dir = os.path.join(settings.MEDIA_ROOT, "thumbs")
                os.makedirs(thumb_dir, exist_ok=True)

                _, extension = os.path.splitext(file_path)
                thumb_path = os.path.join(thumb_dir, f"{self.id}{extension}")

                if os.path.exists(thumb_path):
                    print(f"Thumb for #{self.id} already exists.")
                    return

                if file:
                    pillowImage = Image.open(file)
                else:
                    try:
                        default_storage.open(file_path, 'rb')
                        pillowImage = Image.open(default_storage.open(file_path, 'rb'))
                    except FileNotFoundError:
                        print(f"File {file_path} not found. Deleting the object.")
                        return

                if pillowImage:
                    self.width, self.height = pillowImage.size
                    self.save()

                    if pillowImage.mode != 'RGB':
                        pillowImage = pillowImage.convert('RGB')

                    pillowImage.thumbnail((200, 160))

                    pillowImage.save(thumb_path)
                    print(f"Generated thumbnail for {self.id}.")

        except Exception as e:
            print(f"Error generating thumbnail for {self.id}: {str(e)}\n")



import ast
import json
import os

from django import forms
from django.template.loader import render_to_string
from django.urls import reverse
from django.utils.safestring import mark_safe
from django.contrib.admin import widgets as admin_widgets
from django.conf import settings
from filehub.settings import FILE_TYPE_CATEGORIES


def get_file_size(value):
    try:
        file_path = os.path.join(settings.BASE_DIR, value)
        size = os.path.getsize(file_path)
        if size < 1024:
            return f"{size} B"
        elif size < 1024 * 1024:
            return f"{size / 1024:.2f} KB"
        elif size < 1024 * 1024 * 1024:
            return f"{size / 1024 / 1024:.2f} MB"
        return f"{size / 1024 / 1024 / 1024:.2f} GB"
    except Exception:
        return "Unknown"


class ImagePickerWidget(forms.Textarea):
    """
    A custom widget for selecting images from a file manager.

    This widget renders a textarea for the image URL, and provides a button
    that opens a file manager to select an image. Once an image is selected,
    the URL is inserted into the textarea. The widget also displays the
    selected image's name, size, and type (if it's an image).

    The widget uses the `filehub:browser_select` URL for the file manager
    and allows setting a callback function for handling the image selection.

    This widget is designed to work with Django's admin and forms framework.
    """

    def use_required_attribute(self, *args):
        # The html required attribute may disturb client-side browser validation.
        return False

    def render(self, name, value, attrs=None, renderer=None):
        if attrs is None:
            attrs = {}

        final_attrs = self.attrs.copy()
        final_attrs.update(attrs)

        app_url = reverse('filehub:browser_select')
        filemanager_url = f"{app_url}?callback_fnc={name}"

        file_type = final_attrs.get("data-file-type", "").strip()
        file_ext = final_attrs.get("data-file-ext", "").strip()

        if file_type and not file_ext:
            file_types = [ft.strip() for ft in file_type.split(",") if ft.strip()]
            file_ext_list = []

            for ftype in file_types:
                if not file_ext and ftype in FILE_TYPE_CATEGORIES:
                    file_ext_list.extend(FILE_TYPE_CATEGORIES[ftype])

            if not file_ext and file_ext_list:
                file_ext = ",".join(file_ext_list)

        if file_ext:
            file_exts = [ext.strip() for ext in file_ext.split(",") if ext.strip()]
            filemanager_url += "".join([f"&file_ext={ext}" for ext in file_exts])

        input_field = super().render(name, value, attrs, renderer)
        html = render_to_string("filehub/widgets/image_picker.html", {
            "value": value,
            "name": name,
            "filemanager_url": filemanager_url,
            "basename": os.path.basename(value) if value else None,
            "file_size": get_file_size(value) if value else None,
            "is_image": value and value.endswith((".jpg", ".jpeg", ".png", ".gif", ".svg", ".webp")),
            "input_field": input_field
        })
        return mark_safe(html)

    class Media:
        css = {
            "all": [
                "https://cdnjs.cloudflare.com/ajax/libs/fancybox/3.5.7/jquery.fancybox.min.css",
                f"{settings.STATIC_URL}filehub/widget.css"
            ]
        }
        js = [
            "https://cdnjs.cloudflare.com/ajax/libs/fancybox/3.5.7/jquery.fancybox.min.js",
            f"{settings.STATIC_URL}filehub/widget.js"
        ]


class AdminImagePickerWidget(ImagePickerWidget, admin_widgets.AdminTextareaWidget):
    pass


def is_basename_image(value):
    if not value:
        return False
    ext = os.path.splitext(value)[1].lower()
    if not ext:
        return False

    ext = ext.lstrip('.')
    images_exts = FILE_TYPE_CATEGORIES.get("images", [])
    return ext in images_exts


class BaseFilePickerWidget(forms.Widget):
    default_class = 'file-picker'
    default_value = {}

    class Media:
        css = {
            'all': [
                "https://cdnjs.cloudflare.com/ajax/libs/fancybox/3.5.7/jquery.fancybox.min.css",
                "https://use.hugeicons.com/font/icons.css",
                f"{settings.STATIC_URL}filehub/widget.css"
            ]
        }
        js = [
            "https://cdnjs.cloudflare.com/ajax/libs/fancybox/3.5.7/jquery.fancybox.min.js",
            "https://cdnjs.cloudflare.com/ajax/libs/Sortable/1.15.6/Sortable.min.js",
            f"{settings.STATIC_URL}filehub/widget.js"
        ]

    def __init__(self, attrs=None):
        default_attrs = {'class': self.default_class}
        if attrs:
            default_attrs.update(attrs)
        super().__init__(default_attrs)

    def load_json_value(self, value):
        if value is None:
            return self.default_value

        if isinstance(value, str):
            try:
                return json.loads(value)
            except json.JSONDecodeError:
                return self.default_value
        elif isinstance(value, dict):
            return value
        elif isinstance(value, list):
            return value
        return self.default_value

    def format_value(self, value):
        if value is None:
            return json.dumps(self.default_value)
        if isinstance(value, str):
            if isinstance(value, str):
                try:
                    return json.dumps(json.loads(value))
                except json.JSONDecodeError:
                    try:
                        return json.dumps(ast.literal_eval(value))
                    except (ValueError, SyntaxError):
                        return json.dumps(self.default_value)
        if isinstance(value, dict):
            return json.dumps(value)
        elif isinstance(value, list):
            return json.dumps(value)
        return value

    def get_context(self, name, value, attrs):
        context = super().get_context(name, value, attrs)
        object_value = self.load_json_value(value)
        if not object_value:
            object_value = self.default_value

        context['widget']['object_value'] = object_value
        context['widget']['name'] = name
        context['widget']['value'] = self.format_value(object_value)

        images_exts = FILE_TYPE_CATEGORIES.get("images", [])
        file_manager_url = reverse('filehub:browser_select')
        if self.default_class == 'gallery-picker':
            file_manager_url += f"?callback_fnc={name}&multiple=true"
            if images_exts:
                file_manager_url += f"&file_ext={','.join(images_exts)}"
        else:
            file_name = object_value.get("name", "") if isinstance(object_value, dict) else ""
            context['widget']['is_image'] = is_basename_image(file_name)
            file_manager_url += f"?callback_fnc={name}"
            supported_extensions = context['widget']['attrs'].get("allowed-ext", "")
            if supported_extensions:
                file_manager_url += f"&file_ext={supported_extensions}"

        context['widget']['filemanager_url'] = file_manager_url
        return context


class FilePickerWidget(BaseFilePickerWidget):
    template_name = 'filehub/widgets/file_picker.html'
    default_class = 'file-picker'
    default_value = {}


class GalleryWidget(BaseFilePickerWidget):
    template_name = 'filehub/widgets/gallery_picker.html'
    default_class = 'gallery-picker'
    default_value = []









import json
from django import forms
from django.core.exceptions import ValidationError

from filehub.widgets import GalleryWidget, FilePickerWidget


class FilePickerFormField(forms.JSONField):
    widget = FilePickerWidget

    def __init__(self, *args, **kwargs):
        self.required = kwargs.get("required", False)
        self.file_ext = kwargs.pop("file_ext", [])
        super().__init__(*args, **kwargs)

    def widget_attrs(self, widget):
        attrs = super().widget_attrs(widget)
        if self.file_ext:
            attrs['allowed-ext'] = ','.join(self.file_ext)
        return attrs

    def to_python(self, value):
        if value is None or value == "":
            return {}
        if isinstance(value, str):
            try:
                value = json.loads(value)
            except json.JSONDecodeError:
                value = {}
        return value

    def validate(self, value):
        if self.required and not value.get("url", None):
            raise forms.ValidationError("This field is required.")
        return super().validate(value)


class GalleryFormField(forms.JSONField):
    widget = GalleryWidget

    def __init__(self, *args, **kwargs):
        self.required = kwargs.get("required", False)
        self.min_items = kwargs.pop("min_items", None)
        self.max_items = kwargs.pop("max_items", None)
        self.sortable = kwargs.pop("sortable", False)
        super().__init__(*args, **kwargs)

    def widget_attrs(self, widget):
        attrs = super().widget_attrs(widget)
        if self.min_items is not None:
            attrs['data-min-items'] = str(self.min_items)
        if self.max_items is not None:
            attrs['data-max-items'] = str(self.max_items)
        if self.sortable:
            attrs['sortable'] = 'sortable'
        return attrs

    def to_python(self, value):
        if value is None or value == "":
            return []
        if isinstance(value, str):
            try:
                value = json.loads(value)
            except json.JSONDecodeError:
                value = []
        return value or []

    def validate(self, value):
        if self.required and (not isinstance(value, list) or not value):
            raise forms.ValidationError("Please select at least one image.")

        length = len(value)
        if self.min_items is not None and length < self.min_items:
            raise ValidationError(f"You must select at least {self.min_items} image(s).")
        if self.max_items is not None and length > self.max_items:
            raise ValidationError(f"You can select at most {self.max_items} image(s).")
        return super().validate(value)


import os
import re
from django.conf import settings


def get_ext(filename):
    """
    This function extracts the file extension from a given filename.

    Args:
    filename (str): The name of the file for which the extension needs to be extracted.

    Returns:
    str: The extracted file extension without the dot (.)

    Raises:
    ValueError: If the filename does not contain a valid file extension.

    Example:
    >>> get_ext("example.txt")
    'txt'
    """
    pattern = r"\.(?P<extension>[a-zA-Z0-9]+)$"
    match = re.search(pattern, filename)

    extension_without_dot = ""
    if match:
        extension_without_dot = match.group("extension")

    return extension_without_dot


def sizeof_fmt(num):
    """
    Format file sizes for a human beings.
    https://stackoverflow.com/questions/1094841/reusable-library-to-get-human-readable-version-of-file-size
    """
    for x in ['bytes', 'KB', 'MB', 'GB']:
        if 1024.0 > num > -1024.0:
            return "%3.1f %s" % (num, x)
        num /= 1024.0
    return "%3.1f %s" % (num, 'TB')


def generate_display_name(file_name: str, max_length: int = 15) -> str:
    """
    Generate a display name for a file. If the file name (including the extension)
    exceeds the max_length, it truncates the name and adds '...' in the middle, ensuring
    the total length is max_length.

    :param file_name: Full file name including extension
    :param max_length: Maximum length for the display name
    :return: Truncated display name if necessary
    """
    name, extension = os.path.splitext(file_name)
    if len(file_name) <= max_length:
        return file_name

    remaining_length = max_length - len(extension) - 3
    if remaining_length > 0:
        start = name[:remaining_length // 2]
        end = name[-(remaining_length // 2):]
        return f"{start}...{end}{extension}"
    else:
        return f"...{extension[-(max_length - 3):]}"


def file_response_format(file, request):
    """
    Format file response for the API.
    """
    from filehub.core import FolderManager

    file_full_path = file.get_full_path()
    file_name, file_extension = os.path.splitext(file.file_name)
    thumbnail_url = file_full_path
    if file.file_type == "images" or file.file_type == "image":
        thumbnail_url = str(os.path.join(settings.MEDIA_ROOT, "thumbs", f"{file.id}{file_extension}"))
        thumbnail_url = thumbnail_url.replace(str(settings.BASE_DIR), "")

    file_object = {
        'id': file.id,
        "name": file.file_name,
        "basename": file_name,
        "extension": file_extension,
        "display_name": generate_display_name(file.file_name, 20),
        "type": "file",
        "size": file.file_size,
        "category": file.file_type,
        "uri": file_full_path,
        "url": request.build_absolute_uri(file_full_path),
        "uploaded_at": file.upload_date.strftime("%Y-%m-%d %H:%M:%S"),
        "modify_date": file.modify_date.strftime("%Y-%m-%d %H:%M:%S"),
        "thumbnail": thumbnail_url,
    }

    if file.file_type == "images":
        file_object['width'] = file.width
        file_object['height'] = file.height
    return file_object


def folder_response_format(folder, request):
    """
    Format folder response for the API.
    """
    total_size = folder.get_size()
    file_full_path = folder.get_full_path()
    folder_object = {
        'id': folder.id,
        "name": folder.folder_name,
        "basename": folder.folder_name,
        "display_name": folder.folder_name,
        "type": "folder",
        "size": total_size,
        "category": "folder",
        "uri": file_full_path,
        "url": request.build_absolute_uri(file_full_path),
        "uploaded_at": folder.upload_date.strftime("%Y-%m-%d %H:%M:%S"),
        "modify_date": folder.modify_date.strftime("%Y-%m-%d %H:%M:%S"),
    }
    return folder_object


def get_file_type(file_name):
    """
    Determine the type of a file based on its extension.

    This function uses the `FILE_TYPE_CATEGORIES` configuration from Django settings
    to classify files into categories such as 'images', 'videos', 'musics', or 'archives'.
    If the file's extension does not match any category, it returns 'unknown'.

    Args:
        file_name (str): The name of the file, including its extension.

    Returns:
        str: The category of the file ('images', 'videos', 'musics', 'archives', or 'file').
    """
    _, ext = os.path.splitext(file_name)
    ext = ext.lower().lstrip('.')
    from filehub.settings import FILE_TYPE_CATEGORIES

    for category, extensions in FILE_TYPE_CATEGORIES.items():
        if ext in extensions:
            return category
    return 'file'


def sanitize_allowed_extensions(allowed_extensions):
    """
    Sanitize a list of allowed file extensions by splitting comma-separated strings,
    removing duplicates, trimming whitespace, and converting to lowercase.

    Args:
        allowed_extensions (list): A list of file extensions or comma-separated strings.

    Returns:
        list: A sanitized, sorted list of unique lowercase file extensions.
    """
    if not isinstance(allowed_extensions, list):
        raise ValueError("allowed_extensions must be a list")

    sanitized = set()
    for ext in allowed_extensions:
        if not ext:
            continue
        for part in str(ext).split(","):
            cleaned = part.strip().lower()
            if cleaned:
                sanitized.add(cleaned)

    return sorted(sanitized)








import imghdr
import re
import os
from pathlib import Path

from django.core.exceptions import ValidationError
from django.core.files.storage import default_storage
from django.core.files.base import ContentFile
from django.db import transaction
from filehub import settings
from urllib.parse import urlparse
from filehub.utils import get_ext
from django.conf import settings as main_settings


class FolderManager:

    @staticmethod
    def get_root_directory():
        """
        Returns the root directory for media files.
        """
        return settings.DIRECTORY + "/"

    @staticmethod
    def is_django_default_storage():
        """
        Returns the root directory for media files.
        """
        storages = main_settings.STORAGES.get("default")
        if storages:
            return storages.get("BACKEND") == "django.core.files.storage.FileSystemStorage"
        return False

    @staticmethod
    def get_media_root():
        """
        Returns the root directory for media files.
        """
        return settings.MEDIA_ROOT

    @staticmethod
    def get_media_url():
        """
        Returns the URL for accessing media files.
        """
        return settings.UPLOAD_DIRECTORY_URL

    @staticmethod
    def get_static_url():
        """
        Returns the URL for static files.
        """
        return settings.STATIC_URL

    @staticmethod
    def get_static_root():
        """
        Returns the root directory for static files.
        """
        return settings.STATIC_PATH

    @staticmethod
    def get_file_category(file_path):
        """
        Returns the file type category (image, video, audio, archive, document, or other)
        based on the file's content using `python-magic`.
        """
        file_extension = get_ext(file_path)
        for category, extensions in settings.FILE_TYPE_CATEGORIES.items():
            if file_extension in extensions:
                return category
        return 'files'

    @staticmethod
    @transaction.atomic
    def create_folder(folder_instance):
        """
        Creates a folder in the storage system if it doesn't exist and saves the folder instance.
        """
        try:
            folder_path = folder_instance.get_relative_path()
            if FolderManager.is_django_default_storage():
                os.makedirs(os.path.join(main_settings.MEDIA_ROOT, folder_path), exist_ok=True)
        except Exception as e:
            folder_instance.delete()
            raise Exception(f"Error creating folder: {str(e)}")

    @staticmethod
    @transaction.atomic
    def create_folder_by_path(folder_path: str, parent_folder=None):
        """
        Creates a folder in the storage system if it doesn't exist and saves the folder instance.
        """
        try:
            from filehub.models import MediaFolder
            folder_path = folder_path.strip("/")
            folder_path_obj = folder_path.split("/")
            for folder_name in folder_path_obj:
                if folder_name:
                    parent_folder, _ = MediaFolder.objects.get_or_create(
                        folder_name=folder_name,
                        parent=parent_folder
                    )
            if parent_folder:
                FolderManager.create_folder(parent_folder)
            return parent_folder
        except Exception as e:
            print(f"Error creating folder: {str(e)}")

    @staticmethod
    def delete_orfan_folder(folder_instance):
        """
        Deletes a folder instance without deleting the folder from the storage system.
        """
        try:
            folder_path = folder_instance.get_relative_path()

            def remove_directory(directory):
                directories, files = default_storage.listdir(directory)
                for file in files:
                    try:
                        default_storage.delete(f'{directory}{file}')
                    except Exception:
                        pass

                for directory in directories:
                    remove_directory(f'{directory}{directory}/')

            remove_directory(folder_path)
        except Exception:
            pass

    @staticmethod
    @transaction.atomic
    def delete_folder(folder_instance):
        """
        Deletes a folder from the storage system and removes the folder instance.
        """
        try:
            folder_path = folder_instance.get_relative_path()
            if FolderManager.is_django_default_storage():
                import shutil
                shutil.rmtree(os.path.join(main_settings.MEDIA_ROOT, folder_path))
            else:
                default_storage.delete(folder_path)
        except Exception as e:
            print(f"Error deleting folder: {str(e)}")

    @staticmethod
    @transaction.atomic
    def delete_file(file_instance):
        """
        Deletes a file from the storage system and removes the file instance.
        """

        try:
            file_path = file_instance.get_relative_path()
            default_storage.delete(file_path)
        except Exception as e:
            print(f"Error deleting file: {str(e)}")

    @staticmethod
    @transaction.atomic
    def upload_url(url, folder_instance, filename=None):
        """
        Downloads a file from a URL and uploads it to the storage system.
        """
        import requests
        a = urlparse(url)
        file_name = filename if filename else os.path.basename(a.path)

        if folder_instance:
            folder_path = folder_instance.get_relative_path()
        else:
            folder_path = FolderManager.get_root_directory()

        if FolderManager.is_django_default_storage():
            full_path = os.path.join(main_settings.MEDIA_ROOT, folder_path)
            os.makedirs(full_path, exist_ok=True)

        response = requests.get(url)
        response.raise_for_status()

        image_data = response.content
        detected_format = imghdr.what(None, h=image_data)
        extension = f".{detected_format}" if detected_format else ""

        if extension and not file_name.lower().endswith(extension):
            file_name += extension

        original_file_name, file_extension = os.path.splitext(file_name)
        counter = 1
        file_path = Path(folder_path, file_name)
        while default_storage.exists(file_path.as_posix()):
            file_name = f"{original_file_name}-{counter}{file_extension}"
            file_path = Path(folder_path, file_name)
            counter += 1

        file_size = int(response.headers.get('Content-Length', 0))
        default_storage.save(file_path, ContentFile(image_data))
        return file_size, file_name

    @staticmethod
    @transaction.atomic
    def clean_filename(filename):
        """
        Cleans and sanitizes the filename to ensure it's safe for storage.
        """
        original_file_name, file_extension = os.path.splitext(filename)
        original_file_name = original_file_name.replace(' ', '_')
        original_file_name = original_file_name.replace('-', '_')
        original_file_name = re.sub(r'[^\w\s.-]', '', original_file_name)
        original_file_name = re.sub(r'[\[\](){}]', '', original_file_name)
        return original_file_name + file_extension

    @staticmethod
    @transaction.atomic
    def upload_file(file, folder_instance=None):
        """
        Uploads a file to the storage system, checking for existing files and handling naming conflicts.
        """
        original_file_name, file_extension = os.path.splitext(file.name)
        original_file_name = FolderManager.clean_filename(original_file_name)
        file_name = original_file_name + file_extension

        if folder_instance:
            folder_path = folder_instance.get_relative_path()
        else:
            folder_path = FolderManager.get_root_directory()

        if FolderManager.is_django_default_storage():
            full_path = os.path.join(main_settings.MEDIA_ROOT, folder_path)
            os.makedirs(full_path, exist_ok=True)

        counter = 1
        file_path = Path(folder_path, file_name)
        while default_storage.exists(file_path.as_posix()):
            file_name = f"{original_file_name}-{counter}{file_extension}"
            file_path = Path(folder_path, file_name)
            counter += 1

        with default_storage.open(file_path, 'wb+') as destination:
            for chunk in file.chunks():
                destination.write(chunk)

        file_size = file.size
        return file_size, file_name

    @staticmethod
    @transaction.atomic
    def upload_to_filemanager(file, folder_instance=None, user=None):
        """
        Uploads a file to the storage system and creates a file instance.
        :param file:
        :param folder_instance:
        :return: MediaFile
        """

        from filehub.models import MediaFile

        if isinstance(file, str):
            if not file.startswith("http"):
                raise Exception("Invalid URL")
            file_size, file_name = FolderManager.upload_url(file, folder_instance)
        else:
            file_size, file_name = FolderManager.upload_file(file, folder_instance)

        file_type = FolderManager.get_file_category(file_name)
        return MediaFile.objects.create(
            file_name=file_name,
            folder=folder_instance,
            file_type=file_type,
            file_size=file_size,
            upload_by=user
        )

    @staticmethod
    @transaction.atomic
    def rename_file(previous_instance, updated_instance):
        """
        Renames a file in the storage system and updates the file instance.
        """
        try:
            old_file_path = previous_instance.get_relative_path()
            new_file_path = updated_instance.get_relative_path()

            if default_storage.exists(new_file_path):
                raise Exception("File name already exists")

            if not default_storage.exists(old_file_path):
                raise Exception("File doesn't exist")

            if FolderManager.is_django_default_storage():
                os.rename(
                    os.path.join(main_settings.MEDIA_ROOT, old_file_path),
                    os.path.join(main_settings.MEDIA_ROOT, new_file_path)
                )
            else:
                with default_storage.open(old_file_path, 'rb') as old_file:
                    file_content = old_file.read()
                    default_storage.save(new_file_path, ContentFile(file_content))
                default_storage.delete(old_file_path)
        except Exception as e:
            raise Exception(f"Error renaming file: {str(e)}")



# Generated by Django 5.2.6 on 2025-09-21 16:43

import django.db.models.deletion
from django.conf import settings
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('filehub', '0001_initial'),
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.AddField(
            model_name='mediafile',
            name='upload_by',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, to=settings.AUTH_USER_MODEL),
        ),
        migrations.AddField(
            model_name='mediafolder',
            name='parent',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='filehub.mediafolder'),
        ),
        migrations.AddField(
            model_name='mediafolder',
            name='upload_by',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, to=settings.AUTH_USER_MODEL),
        ),
        migrations.AddField(
            model_name='mediafile',
            name='folder',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='filehub.mediafolder'),
        ),
    ]
